from __future__ import division
import numpy as np
import csv
import os
from os.path import join
import pkg_resources
from utils import get_color_from_intensity, get_color_from_height, \
    create_dir, coors_normalize


class PointvizConverter(object):
    def __init__(self, home):
        self.home = home
        self.task_name = 'default'
        create_dir(self.home, clean=False)
        create_dir(join(self.home, 'data'), clean=False)
        create_dir(join(self.home, 'html'), clean=False)
        return_code = os.system("cp -r {} {}".format(pkg_resources.resource_filename('point_viz', 'src'), self.home))
        if return_code != 0:
            raise IOError("Error occurred when copying html template files, make sure the pip package has been "
                          "properly installed and you have authority to create: {}".format(self.home))

    def convert_pc_csv(self, task_name, coors, default_rgb, intensity):
        coors, norm = coors_normalize(coors)
        if default_rgb is not None:
            assert len(coors) == len(default_rgb)
            rgb = default_rgb
        elif intensity is not None:
            assert len(coors) == len(intensity)
            rgb = get_color_from_intensity(intensity)
        else:
            rgb = get_color_from_height(coors)

        head = ['x', 'y', 'z', 'r', 'g', 'b']
        output_csv = join(self.home, 'data', '{}_pc.csv'.format(task_name))

        with open(output_csv, 'w') as f:
            writer = csv.DictWriter(f, fieldnames=head)
            writer.writeheader()
            for i in range(len(coors)):
                x = coors[i, 0]
                y = coors[i, 1]
                z = coors[i, 2]
                r = rgb[i, 0]
                g = rgb[i, 1]
                b = rgb[i, 2]
                writer.writerow({'x': x, 'y': y, 'z': z, 'r': r, 'g': g, 'b': b})
        return norm

    def convert_bbox_csv(self, task_name, norm, bbox_params, bbox_color=True):
        head = ['label_text', 'color', 'l', 'h', 'w', 'x', 'y', 'z', 'r']
        output_csv = join(self.home, 'data', '{}_bbox.csv'.format(task_name))
        with open(output_csv, 'w') as f:
            writer = csv.DictWriter(f, fieldnames=head)
            writer.writeheader()
            for i in range(len(bbox_params)):
                bbox_param = bbox_params[i]
                assert 7 <= len(bbox_param) <= 9
                l = bbox_param[0] / norm
                h = bbox_param[1] / norm
                w = bbox_param[2] / norm
                x = bbox_param[3] / norm
                y = bbox_param[4] / norm
                z = bbox_param[5] / norm
                r = bbox_param[6]
                color = 'Magenta'
                label_text = ' '
                if len(bbox_param) == 8:
                    if bbox_color:
                        color = bbox_param[7].replace(" ", "").lower()
                    else:
                        label_text = bbox_param[7]
                elif len(bbox_param) == 9:
                    color = bbox_param[7].replace(" ", "").lower()
                    label_text = bbox_param[8]

                writer.writerow({'label_text': label_text,
                                 'color': color,
                                 'l': l, 'h': h, 'w': w,
                                 'x': x, 'y': y, 'z': z,
                                 'r': r})

    def compile(self, task_name, coors, default_rgb=None, intensity=None, bbox_params=None):
        if bbox_params is None:
            self.convert_pc_csv(task_name, coors, default_rgb, intensity)
            os.system("sed 's/TASK_NAME/{}/'; 's/INPUT_PC_CSV/{}/'  "
                      .format(task_name, task_name + '_pc.csv') +
                      "{} > ".format(join(self.home, 'src', 'template.html')) +
                      "{}".format(join(self.home, 'html', task_name + '.html')))
        elif coors is not None:
            norm = self.convert_pc_csv(task_name, coors, default_rgb, intensity)
            self.convert_bbox_csv(task_name, norm, bbox_params)
            os.system("sed  's/TASK_NAME/{}/'; 's/INPUT_PC_CSV/{}/';  's/INPUT_BBOX_CSV/{}/' "
                      .format(task_name, task_name + '_pc.csv', task_name + '_bbox.csv') +
                      "{} > ".format(join(self.home, 'src', 'template.html')) +
                      "{}".format(join(self.home, 'html', task_name + '.html')))
        else:
            norm = np.max(np.abs(bbox_params[:, 3:6]))
            self.convert_bbox_csv(task_name, norm, bbox_params)
            os.system("sed 's/TASK_NAME/{}/'; 's/INPUT_BBOX_CSV/{}/'  "
                      .format(task_name, task_name + '_bbox.csv') +
                      "{} > ".format(join(self.home, 'src', 'template.html')) +
                      "{}".format(join(self.home, 'html', task_name + '.html')))




import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="point_viz_test",
    version="0.0.2",
    author="Zhaoyu Su",
    author_email="zsuad@connect.ust.hk",
    description="A light-weight web point cloud visualizer based on Three.js",
    include_package_data=True,
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/SUZhaoyu/point_viz",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)


from __future__ import division

import numpy as np
import os
from copy import deepcopy

parula_list = [[53, 42, 135],
               [15, 92, 221],
               [18, 125, 216],
               [7, 156, 207],
               [21, 177, 180],
               [89, 189, 140],
               [165, 190, 107],
               [225, 185, 82],
               [252, 206, 46],
               [249, 251, 14],
               [255, 255, 0]]
parula_list = np.array(parula_list, dtype=np.float32)


def get_parula_color_map_from_ratio(ratio):
    parula_idx = ratio * 10.
    if parula_idx <= 0.:
        return parula_list[0]
    elif parula_idx >= 10.:
        return parula_list[-1]
    parula_idx_lower = int(np.floor(parula_idx))
    parula_idx_upper = parula_idx_lower + 1
    parula_rgb_lower = parula_list[parula_idx_lower]
    parula_rgb_upper = parula_list[parula_idx_upper]
    parula_rgb_range = parula_rgb_upper - parula_rgb_lower
    parula_idx_offset = parula_idx - np.floor(parula_idx)
    parula_rgb = parula_rgb_lower + parula_idx_offset * parula_rgb_range
    return np.array([parula_rgb], dtype=np.float32)


def get_rgb_color_map_from_ratio(ratio):
    if 0. <= ratio < 0.25:
        R = 1. - ratio * 2.
        G = ratio * 4.
        B = 0.
    elif 2.5 <= ratio < 0.5:
        R = 0.
        G = 1.
        B = ratio * 4. - 1.
    elif 0.5 <= ratio < 0.75:
        R = 0.
        G = -4. * ratio + 3.
        B = 1.
    else:
        R = 2. * ratio - 1.
        G = 0.
        B = -4. * ratio + 4.
    return np.array([R, G, B], dtype=np.float32) * 255.


cmp_method_dict = {'parula': get_parula_color_map_from_ratio,
                   'rgb': get_rgb_color_map_from_ratio}


def get_color_from_intensity(intensity, cmp='parula'):
    intensity = deepcopy(intensity)
    assert len(intensity.shape) == 1, \
        "The intensity should be in 1-D, but actually got: {}-D".format(len(intensity.shape))
    intensity -= np.percentile(intensity, 1)
    intensity /= np.percentile(intensity, 99)
    colors = np.zeros((intensity.shape[0], 3), dtype=np.float32)
    for i in range(intensity.shape[0]):
        colors[i, :] = cmp_method_dict[cmp](intensity[i])
    return colors


def get_color_from_coors(coors):
    coors = deepcopy(coors)
    assert len(coors.shape) == 2, \
        "The intensity should be in 2-D, but actually got: {}-D".format(len(coors.shape))
    coors -= np.min(coors, axis=0)
    coors /= np.max(coors, axis=0)
    colors = coors * 255.
    return colors


def get_color_from_height(coors, axis=1, cmp='parula'):
    coors = deepcopy(coors)
    height = coors[:, axis]
    height -= np.percentile(height, 1)
    height /= np.percentile(height, 99)
    colors = np.zeros((coors.shape[0], 3), dtype=np.float32)
    for i in range(coors.shape[0]):
        colors[i, :] = cmp_method_dict[cmp](height[i])
    return colors


def get_color_from_class(labels, random_seed=0):
    assert len(labels.shape) == 1, \
        "The labels should be in 1-D, but actually got: {}-D".format(len(labels.shape))
    label_idxs = np.unique(labels)
    colors = np.zeros((labels.shape[0], 3), dtype=np.float32)
    for idx in label_idxs:
        np.random.seed(int(idx + random_seed))
        color = (np.random.uniform(size=[3]) * 255).astype(np.float32)
        label_idx = np.argwhere(labels == idx)
        colors[label_idx, :] = color
    return colors


def create_dir(path, clean=False):
    try:
        os.makedirs(path)
    except OSError:
        if not clean:
            print("WARNING: {} already exists, operation skipped.".format(path))
        else:
            os.system('rm -r {}'.format(path))
            os.makedirs(path)
            print("WARNING: '{}' already exists, and has been cleaned.".format(path))


def coors_normalize(coors):
    norm = np.max(np.percentile(np.abs(coors), 90, axis=0))
    # norm = 1.
    assert norm != 0
    return coors / norm, norm
